%Copyright (C) 2005  Caius-Lavalley

%This program is free software; you can redistribute it and/or
%modify it under the terms of the GNU General Public License
%as published by the Free Software Foundation; either version 2
%of the License, or (at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

% IM = IMRECONSTRUCT(MARKER,MASK) performs morphological reconstruction
%     of the image MARKER under the image MASK.  MARKER and MASK can be two
%     intensity images or two binary images with the same size; IM is an
%     intensity or binary image, respectively.  MARKER must be the same size
%     as MASK, and its elements must be less than or equal to the
%     corresponding elements of MASK
% IM = IMRECONSTRUCT(MARKER,MASK,CONN) performs morphological
%     reconstruction with the specificied connectivity.  CONN may have the
%     following scalar values: 4 or 8 (default)
%     or a specific neighborhood defined by a 3x3 matrix
% Only 2D reconstruction is supported
function [imgRec] = imreconstructdemo (imgMarker, imgMask, conn)


% 1 for Matlab, 0 for Octave
matLab = 1 ;


% Check the number of arguments
if nargin ~= 2 & nargin ~= 3
	error ('Invalid number of arguments : imreconstructdemo <image> <mask> [<connectivity>]')
elseif nargin == 2
    conn = 8 ;
end

% Check the size of images
tmarker = size(imgMarker);
tmask = size(imgMask);

% (number of dimensions)
if length(tmask) ~= length(tmarker)
	error ('Matrix must have the same number of dimensions')
end
% (size of dimensions)
if min(tmarker == tmask) == 0
    error ('Matrix dimensions must be equals')
end
% (dimensions different from 2)
if (length(size(tmask)) ~= 2) | (length(size(tmarker)) ~= 2)
    error ('Matrix dimensions different from 2 not supported')
end


% (the elements of Marker must be less than or equal to the corresponding elements of Mask)
if max(max(imgMarker > imgMask)) > 0
    error ('The elements of Marker must be less than or equal to the corresponding elements of Mask')
end


% Neighborhood definition
% If neighborhood is a number, define corresponding filters
if length(conn) == 1
    if conn == 4
        neighborhood = [0 1 0 ; 1 1 1 ; 0 1 0] ;
    elseif conn == 8
        neighborhood = [1 1 1 ; 1 1 1 ; 1 1 1] ;
    else
        error ('The number of neighbors must be 4 or 8 or a 3x3 matrix') ;
    end
% If neighborhood is a matrix, check its dimensions
else
    tConn = size(conn) ;
    if length(tConn) == 2 & tConn(1) == 3 & tConn(2) == 3
        neighborhood = conn ;
    else    
        error ('The number of neighbors must be 4 or 8 or a 3x3 matrix') ;
    end
end

% Neighborhood filters for the two iterations
% (two directions)
neighbFilter1 = [1 1 1 ; 1 1 0 ; 0 0 0] ;
neighbFilter2 = [0 0 0 ; 0 1 1 ; 1 1 1] ;

% Separate the neighborhood filters in two parts
neighb1 = neighborhood .* neighbFilter1 ;
neighb2 = neighborhood .* neighbFilter2 ;


% Mark the result image
%%	Suppr Octave
if matLab
    imgMarker = double(imgMarker) ;
end
imgRec = imgMarker ;

% Propagate the markers
nl = tmask(1);
nc = tmask(2);



% Add a black border to image to avoid particular operations on borders
nl = nl+2 ;
nc = nc+2 ;
% Enlarge the image
imgMask2 = zeros(nl,nc) ;
imgMask2(2:nl-1,2:nc-1) = imgMask ;
imgMask = imgMask2 ;
% Enlarge the mask
imgRec2 =  zeros(nl,nc) ;
imgRec2(2:nl-1,2:nc-1) = imgRec ;
imgRec = imgRec2 ;


modif = 1 ;

while modif ~= 0
	% (propagation until there is no more modification)
	modif = 0 ;

    % Upper left -> Bottom right
	for y = 2 : nl-1
        for x = 2 : nc-1
            % Check neighbors
            neighbors = imgRec(y-1:y+1,x-1:x+1) .* neighb1 ;
            v = min(imgMask(y,x),max(max(neighbors))) ;
            if v ~= imgRec(y,x)
                imgRec(y,x) = v ;
                modif = 1 ;
            end
		end
	end
	
	% Bottom right -> Upper left
	for y = nl-1 : -1 : 2
        for x = nc-1 : -1 : 2
            % Check neighbors
            neighbors = imgRec(y-1:y+1,x-1:x+1) .* neighb2 ;
            v = min(imgMask(y,x),max(max(neighbors))) ;
            if v ~= imgRec(y,x)
                imgRec(y,x) = v ;
                modif = 1 ;
            end
		end
	end

end

% Resize the image
imgRec = imgRec(2:nl-1,2:nc-1) ;

%%	Suppr Octave
if matLab
    imgRec = uint8(imgRec) ;
end

%% Octave (supported by MatLab)
end